from django.contrib.auth.mixins import (LoginRequiredMixin,
                                        PermissionRequiredMixin)
from django.views.generic.edit import (BaseCreateView, BaseUpdateView,
                                       BaseDeleteView)
from django.contrib.auth import get_permission_codename

from ldb.django.contrib.auth.permissions import get_permission_full_codename

class OwnedObjectQuerysetMixin(LoginRequiredMixin):
    """
    CBV mixin which filters the queryset based on owner.

    Assumes that the model is an OwnedModel, or at least that it has an owner
    foreign key to the auth user.
    """
    def get_queryset(self):
        queryset = super(OwnedObjectQuerysetMixin, self).get_queryset()
        return queryset.filter(owner=self.request.user)


class OwnedObjectFormMixin(LoginRequiredMixin):
    """
    CBV mixin that adds the logged in user's instance to the created object.
    """
    def form_valid(self, form):
        form.instance.owner = self.request.user
        return super(OwnedObjectFormMixin, self).form_valid(form)


class AuthorizedMixin(PermissionRequiredMixin):
    """
    CBV mixin that makes views require either a permission based on their type.
    Default is view, BaseCreateView subclasses require add, BaseUpdateView
    subclasses require change and BaseDeleteView subclasses required delete.
    """
    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)

        # This appears to be an abstract version of some CBV
        # If this is a bad assumption, then something needs to be done about
        # cls.model._meta below
        if cls.model is None:
            return

        permission_action = 'view'

        if hasattr(cls, 'permission_action'):
            permission_action = cls.permission_action
        elif issubclass(cls, BaseCreateView):
            permission_action = 'add'
        elif issubclass(cls, BaseUpdateView):
            permission_action = 'change'
        elif issubclass(cls, BaseDeleteView):
            permission_action = 'delete'

        cls.permission_required = \
                get_permission_full_codename(cls.model, permission_action)



from ldb.django.contrib.auth.mixins import AuthorizedMixin

from django.views.generic import (ListView, DetailView, CreateView, UpdateView,
                                  DeleteView)

#TODO: Test these
class AuthorizedListView(AuthorizedMixin, ListView): pass
class AuthorizedDetailView(AuthorizedMixin, DetailView): pass
class AuthorizedCreateView(AuthorizedMixin, CreateView): pass
class AuthorizedUpdateView(AuthorizedMixin, UpdateView): pass
class AuthorizedDeleteView(AuthorizedMixin, DeleteView): pass



from django.contrib.auth.decorators import permission_required, login_required
from django.conf.urls import url


def permission_required_url(permission, regex, view, *args, **kwargs):
    return url(regex,
               permission_required(permission, raise_exception=True)(view),
               *args, **kwargs)

def login_required_url(regex, view, *args, **kwargs):
    return url(regex, login_required(view), *args, **kwargs)



import tempfile
import os

from django.test.runner import DiscoverRunner
import django
from django.conf import settings

from ldb.django.test.testcases import DjangoAppTestCaseMixin


def get_apps(test_or_suite):
    apps = set()
    if isinstance(test_or_suite, DjangoAppTestCaseMixin):
        apps.update(set(test_or_suite.get_applications()))
    else:
        for sub_test_or_suite in test_or_suite:
            apps.update(get_apps(sub_test_or_suite))

    return apps


class DjangoTestRunner(DiscoverRunner):
    def run(self, test):
        tmp_file = tempfile.NamedTemporaryFile(delete=False)
        tmp_filename = tmp_file.name
        tmp_file.close()

        installed_apps = list(get_apps(test))
        settings.configure(DEBUG=True,
                           DATABASES={
                               'default': {
                                   'ENGINE': 'django.db.backends.sqlite3',
                                   'NAME': tmp_filename,
                               }
                           },
                           INSTALLED_APPS=installed_apps,
                           )

        django.setup()

        self.setup_test_environment()
        old_config = self.setup_databases()

        results = self.run_suite(test)

        self.teardown_databases(old_config)
        self.teardown_test_environment()

        os.remove(tmp_filename)

        return results



import os
import sys

import django
from django.conf import settings
from django.core.management import call_command

def makemigrations():
    sys.path.insert(0, os.getcwd())
    settings.configure(DEBUG=True,
                       DATABASES={
                           'default': {
                               'ENGINE': 'django.db.backends.sqlite3',
                           }
                       },
                       SECRET_KEY='fake_key',
                       INSTALLED_APPS=[sys.argv[1]],
                      )

    django.setup()
    call_command('makemigrations', sys.argv[1], skip_checks=False)



from django.contrib.auth import get_permission_codename


def get_permission_full_codename(model_cls, permission_action):
    """
    model_cls is the class of a model
    permission_action is the permission like {view, add, change, delete} (as a
        string)
    """
    permission_required_code = \
                get_permission_codename(permission_action, model_cls._meta)
    return f"{model_cls._meta.app_label}.{permission_required_code}"



class InitialKwargsMixin:
    """
    Use this by using .as_view(initial_kwargs={'form_key1': 'kwarg_key1',
                                               'form_key2': 'kwarg_key2'})
    """
    initial_kwargs = {}

    def get_initial(self):
        initials = super().get_initial()

        for form_key, kwargs_key in self.initial_kwargs.items():
            initials[form_key] = self.kwargs[kwargs_key]

        return initials



from ldb.django.test.testcases import DjangoAuthAppTestCaseMixin
from django.test.testcases import TransactionTestCase


class ModelTestCase(DjangoAuthAppTestCaseMixin, TransactionTestCase):
    applications = ['test_app', 'ldb.django.contrib.migration']
    root_urlconf = ''

    def getTestModel(self, state):
        from django.db.migrations.executor import MigrationExecutor
        from django.db import connection

        executor = MigrationExecutor(connection)
        old_state = [('test_app', state)]
        old_apps = executor.loader.project_state(old_state).apps
        executor.migrate(old_state)
        TestModel = old_apps.get_model('test_app', 'TestModel')

        return TestModel

    def setUp(self):
        TestModel = self.getTestModel('0002_testmodel_test2')
        TestModel.objects.create(test='test', test2='test2')
        a = TestModel(test='test_2', test2='test2_2')
        a.save()

    def test_a(self):
        from test_app.models import TestModel
        self.getTestModel('0003_move_model_add_test3')

        self.assertEqual(TestModel.objects.get(test='test').test, 'test')

        a = TestModel(test='hello', test3='asdf')
        a.save()

    def test_b(self):
        from test_app.models import TestModel
        self.getTestModel('0003_move_model_update')

        self.assertEqual(TestModel.objects.get(test='test').test, 'test')
        self.assertEqual(TestModel.objects.get(test='test').test3, 'test2')
        self.assertEqual(TestModel.objects.get(test='test').test3, 'test2')

        a = TestModel(test='hello', test3='asdf')
        a.save()

    def test_c(self):
        from test_app.models import TestModel
        self.getTestModel('0003_move_model_remove_test2')

        self.assertEqual(TestModel.objects.get(test='test').test, 'test')
        self.assertEqual(TestModel.objects.get(test='test').test3, 'test2')
        self.assertEqual(TestModel.objects.get(test='test').test3, 'test2')

        a = TestModel(test='hello', test3='asdf')
        a.save()



import json

from django import template

from ldb.django.forms import json as form_json_

register = template.Library()

@register.simple_tag
def form_json(form):
    return json.dumps(form_json_.json_from_form(form))



from django.forms import widgets

def json_from_form(form):
    json_obj = {}
    form_field_name_order = form.fields.keys()

    json_obj['fields'] = []
    json_obj['multipart'] = form.is_multipart()

    for field_name in form_field_name_order:
        bound_field = form[field_name]
        widget = bound_field.field.widget

        field_obj = {'label': bound_field.label,
                     'name': bound_field.name,
                     'type': widget.__class__.__name__,
                     'id': bound_field.auto_id,
                     'attrs': widget.attrs}

        if isinstance(widget, widgets.Input):
            field_obj['input_type'] = widget.input_type

        if isinstance(widget, widgets.Select):
            field_obj['choices'] = [[str(_) for _ in a]
                                    for a in widget.choices]

        json_obj['fields'].append(field_obj)

    return json_obj



from django.db import models
from django.contrib.auth.models import User

class OwnedModel(models.Model):
    owner = models.ForeignKey(User, on_delete=models.PROTECT)

    class Meta:
        abstract = True



from setuptools import setup

setup(name='LDB_Django_Common',
      version='0.0.18',
      description="Common utility functions and classes for django.",
      author='Alex Orange',
      author_email='alex@eldebe.org',
      packages=['ldb', 'ldb.django', 'ldb.django.test', 'ldb.django.contrib',
                'ldb.django.contrib.auth', 'ldb.django.db',
                'ldb.django.db.migrations', 'ldb.django.contrib.migration',
                'ldb.django.conf', 'ldb.django.forms',
                'ldb.django.forms.templatetags',
               ],
      include_package_data=True,
      namespace_packages=['ldb', 'ldb.django'],
      url='http://www.eldebe.org/ldb/django/common/',
      license='BSD',
      setup_requires=['setuptools_hg'],
      install_requires=['Django>=1.9.1'],
      test_runner='ldb.django.test.runner:DjangoTestRunner',
      test_suite='tests',
      entry_points={
          'console_scripts': [
              'django-makemigrations = ldb.django.db.migrations.makemigrations:makemigrations',
          ],
      }
     )



from __future__ import unicode_literals

from django.db import models

# Create your models here.
class TestModel(models.Model):
    test = models.CharField(max_length=255)
    test3 = models.CharField(max_length=255)

    def __unicode__(self):
        return "test: %s test3: %s"%(test, test3)
